==> toolbelt-spec-0.1.5-20180328.224057-1.jar: toolbelt/spec.clj <==
(ns toolbelt.spec
  "A `spec` library of those commonly used."
  (:require [clojure.spec.alpha :as s]
            [toolbelt.async :as ta]))

;; ==============================================================================
;; spec =========================================================================
;; ==============================================================================


;; async helpers ========================


(defn channel
  "Takes a spec and returns a spec for a channel.
  The inner spec is ignored, and used just for documentation purposes."
  ([] (channel any?))
  ([spec] ta/chan?))


(defn async
  "Takes a spec and returns either a spec for the passed-in
  inner spec OR a channel.

  If the Stripe method called is async, the inner spec is ignored and
  used just for documentation purposes. If not, the inner spec is used."
  ([] (async any?))
  ([spec]
   (s/or :spec spec :channel (channel spec))))


;; math helpers =========================


(defn between
  "Returns a predicate that checks that the supplied number falls
  between the inclusive lower and exclusive upper bounds supplied."
  [low high]
  (fn [x]
    (and (>= x low)
         (< x high))))


;; TODO move this elsewhere?
;; (defn collectify [x]
;;   #?(:cljs
;;      (if (sequential? x) x [x])
;;      :clj
;;      (cond (nil? x) []
;;            (or (sequential? x) (instance? java.util.List x) (set? x)) x
;;            :else [x])))


;; time helpers ========================


(defn now [] (new java.util.Date))

(defn year-now [] (.getValue (java.time.Year/now)))

(defn month-now [] (.getValue (.getMonth (java.time.YearMonth/now))))


;; basic specs ==========================


(s/def ::description (s/nilable string?))

(defn description? [x] (s/valid? ::description x))

(s/def ::pos-float
  (s/and pos? float?))

(defn pos-float? [x] (s/valid? ::pos-float x))

(s/def ::pos-float-incl-zero
  (s/or :zero zero? :pos-float pos-float?))

(defn pos-float-incl-zero? [x] (s/valid? ::pos-float-incl-zero x))

(s/def ::neg-float
  (s/and neg? float?))

(defn neg-float? [x] (s/valid? ::neg-float x))

(s/def ::exp-month
  (s/and pos-int? (between 1 12)))

(defn exp-month [x] (s/valid? ::exp-month x))

(s/def ::exp-year
  #(<= (year-now) %))  ;; TODO if = then check that exp-month is greater than month-now

(defn exp-year [x] (s/valid? ::exp-year x))


;; email ================================


(def email-regex #"^[a-zA-Z0-9._%+-]+@[a-zA-Z0-9.-]+\.[a-zA-Z]{2,63}$")

(s/def ::email (s/and string? #(re-matches email-regex %)))

(defn email? [x] (s/valid? ::email x))


;; timestamp ============================


(s/def ::unix-timestamp
  integer?)

(defn unix-timestamp? [ts]
  (s/valid? ::unix-timestamp ts))

==> toolbelt-spec-0.1.5-20180330.053933-2.jar: toolbelt/spec.clj <==
(ns toolbelt.spec
  "A `spec` library of those commonly used."
  (:require [clojure.spec.alpha :as s]
            [toolbelt.async :as ta]))

;; ==============================================================================
;; spec =========================================================================
;; ==============================================================================


;; async helpers ========================


(defn channel
  "Takes a spec and returns a spec for a channel.
  The inner spec is ignored, and used just for documentation purposes."
  ([] (channel any?))
  ([spec] ta/chan?))


(defn async
  "Takes a spec and returns either a spec for the passed-in
  inner spec OR a channel.

  If the Stripe method called is async, the inner spec is ignored and
  used just for documentation purposes. If not, the inner spec is used."
  ([] (async any?))
  ([spec]
   (s/or :spec spec :channel (channel spec))))


;; math helpers =========================


(defn between
  "Returns a predicate that checks that the supplied number falls
  between the inclusive lower and exclusive upper bounds supplied."
  [low high]
  (fn [x]
    (and (>= x low)
         (< x high))))


;; TODO move this elsewhere?
;; (defn collectify [x]
;;   #?(:cljs
;;      (if (sequential? x) x [x])
;;      :clj
;;      (cond (nil? x) []
;;            (or (sequential? x) (instance? java.util.List x) (set? x)) x
;;            :else [x])))


;; time helpers ========================


(defn now [] (new java.util.Date))

(defn year-now [] (.getValue (java.time.Year/now)))

(defn month-now [] (.getValue (.getMonth (java.time.YearMonth/now))))


;; basic specs ==========================


(s/def ::description (s/nilable string?))

(defn description? [x] (s/valid? ::description x))

(s/def ::pos-float
  (s/and pos? float?))

(defn pos-float? [x] (s/valid? ::pos-float x))

(s/def ::pos-float-incl-zero
  (s/or :zero zero? :pos-float pos-float?))

(defn pos-float-incl-zero? [x] (s/valid? ::pos-float-incl-zero x))

(s/def ::neg-float
  (s/and neg? float?))

(defn neg-float? [x] (s/valid? ::neg-float x))

(s/def ::exp-month
  (s/and pos-int? (between 1 12)))

(defn exp-month? [x] (s/valid? ::exp-month x))

(s/def ::exp-year
  #(<= (year-now) %))  ;; TODO if = then check that exp-month is greater than month-now

(defn exp-year? [x] (s/valid? ::exp-year x))


;; email ================================


(def email-regex #"^[a-zA-Z0-9._%+-]+@[a-zA-Z0-9.-]+\.[a-zA-Z]{2,63}$")

(s/def ::email (s/and string? #(re-matches email-regex %)))

(defn email? [x] (s/valid? ::email x))


;; timestamp ============================


(s/def ::unix-timestamp
  integer?)

(defn unix-timestamp? [ts]
  (s/valid? ::unix-timestamp ts))
